function Foveon2OtherFormat_batch(origpath, destinpath, method, downsize, IRfilter, makejpg, showpic)

% Default program to convert a batch of files from Foveon camera rgb to
% any other format (see list). input parameters are:

% showpic :  whether it shows a suitably scaled version of the picture being
%            converted (the default is NO)
% makejpg :  whether it makes a pretty jpg version of the picture being
%            processed in a subsidiary directory called "/pics" (the
%            default is YES)
% IRfilter : whether the IR-blocking filter was used in during the image
%            capture (used for bakwards compatibility with databases
%            gathered WITHOUT the filter) (the default is FILTER ON)
% downsize : since the original foveon files are quite big (2268 x 1512) you are given
%            the option of downsizing. The default is downsize = 0 (NO changes)
%            another popular option is downsize = 0.5 (half-size).
% method   : this determines the choice of polynomial coversion.
%            There are seveal flavours to date (see the test_all_poly_options,
%            but the most common use of this code is to convert the pictures to 
%            the Stockmann & Sharpe LMS fundamentals using the datset of Munsell 
%            chip reflectances as a training set (method = SS_LMS_Munsell)
%            or to convert them to the standard CIE1931 XYZ tristimulus
%            values using the datset of Munsell chip reflectances as a training 
%            set (method = CIE1931_XYZ_Munsell). The complete set of options are: 
%            SP_LMS_Munsell: Smith & Pokorny LMS (normalised to max=1) and Munsell chips 
%            SP_LMS_NE     : Smith & Pokorny LMS (normalised to max=1) and NE reflectances
%            SS_LMS_Munsell: Stockmann & Sharpe 2000 LMS (normalised to max=1) and Munsell chips
%            SS_LMS_NE     : Stockmann & Sharpe 2000 LMS (normalised to max=1) and NE reflectances 
%            CIE1931_XYZ_Munsell : CIE1931 XYZ (normalised to max=1) and Munsell chips
%            CIE1931_XYZ_NE      : CIE1931 XYZ (normalised to max=1) and NE reflectances
%            JV1978_XYZ_Munsell  : Judd-Voss 1978 XYZ (normalised to max=1) and Munsell chips
%            JV1978_XYZ_NE       : Judd-Voss 1978 XYZ (normalised to max=1) and NE reflectances
%origpath  : where the original pictures are located. BEWARE! it will
%            process ALL .x3f in that directory
%destinpath: where the processed files will go. By default is the same as origpath. 
%===============================

if (nargin < 7)  || isempty(showpic)
    showpic = 0;
end
if (nargin < 6)  || isempty(makejpg)
    makejpg = 1;
end
if (nargin < 5)  || isempty(IRfilter)
    IRfilter = 1;
end
if (nargin < 4)  || isempty(downsize)
    downsize = 0;
end
if (nargin <3)  || isempty(method)
    method = 'cie1931_xyz_munsell';
end
if (nargin <2)  || isempty(destinpath)
    destinpath = origpath;
end
if (nargin <1)  || isempty(origpath)
    error( 'Need to enter the destination path to the images.')
end

destinpathjpg = fullfile(destinpath,'pics\');
destinpath_finished = fullfile(destinpath, '\', method, '\');
foveonfiles    = dir(fullfile(origpath,'*.X3F'));
foveonlength = length(foveonfiles);

jpgsize = [378 567]; %Size of the output jpg pic sample (if any)

[M SumSQResid] = generate_polyparam(method, IRfilter, showpic);

for i= 1:foveonlength    
    testpic= fullfile(origpath,foveonfiles(i,1).name);
    h = read_x3f_header(testpic);
    f_number = str2double(h.APERTURE);
    integrationtime = str2double(h.SHUTTER);
    disp('=========================================');
    disp(['Processing image ' foveonfiles(i,1).name]);
    
    foveon = double(readfoveonpic(testpic));
    if downsize
        foveon = imresize(foveon, downsize);
    end
    if showpic
        imagesc(foveon./max(foveon(:)));
    end
    foveon_processed = fovRGB2OtherFormat(foveon, M, f_number, integrationtime);   
    myjunk= strrep(testpic, origpath, destinpath_finished);
    myjunk= regexprep(myjunk, '.X3F', '.mat');
    if ~exist(fileparts(myjunk), 'dir')
        mkdir(fileparts(myjunk));
    end
    save(myjunk, 'foveon_processed');
    
    if makejpg
        disp('=========================================');
        disp(['Making jpg image ' foveonfiles(i,1).name]);
         myjunk= strrep(testpic, origpath, destinpathjpg);
         myjunk= regexprep(myjunk, '.X3F', '.jpg' );
         if ~exist(myjunk, 'file')
             jpgfoveon = double(readfoveonpic(testpic, '-h -a'));
             jpgfoveon = imresize(jpgfoveon, jpgsize);
             jpgfoveon = (jpgfoveon-min(jpgfoveon(:))) ./ (max(jpgfoveon(:))-min(jpgfoveon(:)));

            if ~exist(fileparts(myjunk), 'dir');
                mkdir(fileparts(myjunk));
            end                
            imwrite(jpgfoveon,  myjunk, 'jpg');
         else
            disp('jpg image already exists: skipping'); 
         end
    end
end



disp('All done');
