function ave_out = ave_images(dname, ext, Ngroup, index, trm, vis)

% AVE_IMAGES   - Calculates and displays average images for each
%                exposure level, both with and without trend
%
% Function call:   ave_out = ave_images(dname,ext,Ngroup,index,trm,vis);
%
% In:   dname       Directory name for data files
%       ext         file name extension (e.g. 'x3f')
%       Ngroup      Number of files in one exposure group (default = 10)
%       index       Index numbers for exposure levels to be averaged
%                   (default all)
%       trm         trend removal; 1 = yes, 0 = no (default 0)
%       vis         result plotting; 1 = yes, 0 = no (default 0, use
%                   plot_ave_images for results)
%
% Out:  ave_out     cell array containg:
%                   ch_ave1, ch_ave2 etc: 3D matrix of color channel averages with trend
%                   Tch_ave1, Tch_ave2 etc: trendless averages for color
%                   channels

% (c) Heli Hytti 3.2.2005
% HH 22.2.2006: arbitrary number of color channels

% Add backslash to the end of directory name if it's missing
if dname(end)~='\', dname = [dname '\']; end

% Get file names in time order (monotonically increasing exposure level
% assumed)
F = files(dname,ext,'t');

% Find number of image files and calculate number of exposure levels
Ns = size(F,2);
Nexp = Ns/Ngroup;

% Set default parameters if needed:
% Result plotting off
if nargin<6, vis = 0; end
% Trend removal off
if nargin<5, trm = 0; end
% Calculate average images for all exposure levels
if nargin<4, index = 1:Nexp; end
% Default number of frames per exposure level is 10
if nargin<3, Ngroup = 10; end

% Read first image
I = readraw([dname F{index(1)}],'O');
nw = size(I,2); nh = size(I,1); nc = size(I,3);

% Initialize output matrices
for kk=1:nc, eval(['ch' num2str(kk) ' = zeros(nh,nw);']);
  eval(['ch_ave' num2str(kk) ' = zeros(nh,nw,length(index));']);
end

if trm==1,
    % Trend removal filter
    [ba, aa] = besself(2, 0.01*2*pi); [b, a] = impinvar(ba, aa, 1);
    X = zeros(1001); X(501,501)=1; H = filtsk(X, b, a); sH = sum(H(:));
    H(:) = H/sH; b = b/sH;

    for kk=1:nc, eval(['Tch_ave' num2str(kk) ' = zeros(nh,nw,length(index));']); end
end

% For each required exposure level repeat:
h = waitbar(0,'Please wait... Processing data');
for ii=1:length(index),
    for kk=1:nc, eval(['chans' num2str(kk) ' = zeros(nh,nw);']); end
    if trm==1, for kk=1:nc, eval(['Tchans' num2str(kk) ' = zeros(nh,nw);']); end; end

    % Find appropriate data file index (first file in given exposure level)
    k = (index(ii)-1)*Ngroup+1;
    % For Ngroup files repeat
    for jj=1:Ngroup,
        % Read images
        I(:) = readraw([dname F{k}],'O');
        % Separate color channels, calculate signal sums and trendless
        % signal sums
        for kk=1:nc, eval(['ch' num2str(kk) ' = double(I(:,:,kk));']);
          eval(['chans' num2str(kk) '(:) = chans' num2str(kk) '+ch' num2str(kk) ';']);
          if trm==1, eval(['Tchans' num2str(kk) '(:) = Tchans' num2str(kk) '+(ch' num2str(kk) ' - filtsk(ch' num2str(kk) ',b,a));']); end
        end
        % Calculate signal sums and trendless signal sums
        k = k+1;
    end
	
    % Calculate average images, both with and without trend
    for kk=1:nc, eval(['ch_ave' num2str(kk) '(:,:,ii) = chans' num2str(kk) '/Ngroup;']);    
      if trm==1, eval(['Tch_ave' num2str(kk) '(:,:,ii) = Tchans' num2str(kk) '/Ngroup;']); end
    end
    waitbar(ii/length(index)),
end
close(h);

if vis==1,
  if nc==1, cnames = 'grayscale';
  elseif nc==3, cnames = str2mat('red','green','blue');
  elseif nc==4, cnames = str2mat('red','g-red','blue','g-blue');
  end
  for kk=1:nc,
    figure; eval(['h = immulti(ch_ave' num2str(kk) ');']); title(['Average ' deblank(cnames(kk,:)) ' components at each exposure level']);
    if trm==1,
       figure; eval(['h = immulti(Tch_ave' num2str(kk) ');']); title(['Average trendless ' deblank(cnames(kk,:)) ' components at each exposure level']);
    end
  end
end

if trm==0,
  ave_out = cell(1,nc);
  for ii=1:nc, ave_out(ii) = eval(['{ch_ave' num2str(ii) '};']); end
else
  ave_out = cell(1,2*nc);
  for ii=1:nc, ave_out(ii) = eval(['{ch_ave' num2str(ii) '};']); end
  for ii=nc+1:2*nc, ave_out(ii) = eval(['{Tch_ave' num2str(ii-nc) '};']); end
end
