function out = dc_shotnoise(dname, ext, Ngroup, index, ave_in)

% DC_SHOTNOISE   - Analyses dark current shot noise for different exposure
%                 levels
%
% Function call: out = dc_shotnoise(dname,ext,Ngroup,index,ave_in);
%
% In:   dname       Directory name for data files
%       ext         file name extension (e.g. 'x3f')
%       Ngroup      Number of files in one exposure group (default = 100)
%       index       Index numbers for exposure levels to be averaged
%                   (default all)
%       ave_in      cell array (output from ave_images) containing 
%                   ave_ch1, ave_ch2 etc; average dark images for color components 
%                   for each exposure level in matrix
%
% Out:  out         cell array containing shot_ch1, shot_ch2 etc; 
%                   3D matrices of pixel-by-pixel dc shot noise variances

% (c) Heli Hytti 25.4.2005
% HH 23.2.2006: arbitrary number of color channels

% Default number of frames per exposure level is 100
if nargin<3, Ngroup = 100; end

% Add backslash to the end of directory name if it's missing
if dname(end)~='\', dname = [dname '\']; end

% Get file names in time order (monotonically increasing exposure level
% assumed)
F = files(dname,ext,'t');

% Find number of image files and calculate number of exposure levels
Ns = size(F,2);
Nexp = Ns/Ngroup;

% Set default parameters if needed:

% Calculate average images for all exposure levels
if nargin<4, index = 1:Nexp; end

% Read test image and get image size
I = readraw([dname F{1}],'O');
nw = size(I,2); nh = size(I,1); nc = size(I,3);

% Initialize matrices
for ii=1:nc, eval(['shot_ch' num2str(ii) ' = zeros(nh,nw,length(index));']); 
  eval(['ch' num2str(ii) ' = zeros(nh,nw);']); 
end
I = zeros(nh,nw,nc);

% For each required exposure level repeat:
for ii=1:length(index),
    for kk=1:nc, eval(['chans' num2str(kk) ' = zeros(nh,nw);']); end
    % Find appropriate data file index (first file in given exposure level)
    k = (index(ii)-1)*Ngroup+1;
    % For Ngroup files repeat
    for jj=1:Ngroup,
        % Read images
        I = readraw([dname F{k}],'O');
        % Separate color channels and calculate signal sums
        for kk=1:nc,
          eval(['ch' num2str(kk) '(:) = double(I(:,:,kk));']);
          eval(['chans' num2str(kk) '(:) = chans' num2str(kk) ' + ch' num2str(kk) '.^2;']);
        end
        k = k+1;
    end
	
    % Calculate average image variances
    for kk=1:nc,
      ave_ch = ave_in{kk};
      eval(['shot_ch' num2str(kk) '(:,:,ii) = (chans' num2str(kk) '-Ngroup*ave_ch(:,:,index(ii)).^2)/(Ngroup-1);']);
    end
end

out = cell(1,nc);
for ii=1:nc, out(ii) = eval(['{shot_ch' num2str(ii) '};']); end
