function [s_ave, var_diff, s_temp, s_fp, mC, SNR_temp, PRNU, PRNU_lev, out] = noise_analysis(dname, ext, exposure, offset, smax, Ngroup)

% NOISE_ANALYSIS    - Calculates noise components according to ISO standard
%                     ISO 15739
%
% Function call:    
%   [s_ave,var_diff,s_temp,s_fp,mC,SNR_temp,PRNU,PRNU_lev,out] =
%                noise_analysis(dname,ext,exposure,offset,smax,Ngroup);
%
% In:   dname       Directory name for data files
%       ext         file name extension (e.g. 'x3f')
%       exposure    Input exposure values [lux*s] or scene log luminance
%                   values [cd*s/m^2]
%       offset      camera offset level
%       smax        maximum signal level for camera
%       Ngroup      Number of files in one exposure group (default = 10)
%
% Out:  s_ave      standard deviations of average color components (columns)
%       var_diff   means of variances of differenced color components (columns)
%       s_temp     standard deviations of temporal noise for color components
%       s_fp       standard deviations of the fixed pattern noise
%       mC         mean values of average color components at each exposure level
%       SNR_temp   signal-to-temporal noise ratio at each exposure level
%       PRNU       photo-response non-uniformity
%       PRNU_lev   photoresponse non-uniformity determination level
%       out        cell array containing:
%                  ave_ch1, ave_ch2 etc; 3D-matrices of average color component values (one for each exposure level, 
%                  64 x 64 matrix from the middle of the image)
%                  std_ch1, std_ch2 etc: pixel-by-pixel STDs of color components at each exposure level

% (c) Heli Hytti 22.2.2005
% HH 22.2.2006: arbitrary number of color channels

% Set default parameters if needed:
% Default number of frames per exposure level is 10
if nargin<6, Ngroup = 10; end

% Add backslash to the end of directory name if it's missing
if dname(end)~='\', dname = [dname '\']; end

% Find index for minimum exposure level (dark frames)
[minexp,min_ind] = min(exposure);

% Get file names in time order (monotonically increasing exposure level
% assumed)
F = files(dname,ext,'t');

% Find number of image files and calculate number of exposure levels
Ns = size(F,2);
Nexp = Ns/Ngroup;

% Define size of pixel area in the middle of the image for analysis
pixarea = [64 64];

% Input log illuminance/exposure or scene log luminance
val = log10(exposure);

% Get image size and calculate coordinates for 64 x 64 pixel area in the middle
% of the image
% Read test image
I = readraw([dname F{1}],'O');
nw = size(I,2); nh = size(I,1); nc = size(I,3);
area_x = round((nw/2)-31):round((nw/2)+32); 
area_y = round((nh/2)-31):round((nh/2)+32);

% Initialize file index k
k = 1; 

% Initialize output matrices
for kk=1:nc, eval(['ave_ch' num2str(kk) ' = zeros(64,64,Nexp);']);
  eval(['std_ch' num2str(kk) ' = zeros(64,64,Nexp);']);
end
s_ave = zeros(Nexp,nc); var_diff = s_ave; s_temp = s_ave; s_fp = s_ave;
mC = zeros(Nexp,nc); SNR_temp = mC; PRNU = zeros(1,nc);

% Initialize matrices for calculations in loop
for kk=1:nc, eval(['chans' num2str(kk) ' = zeros(64,64,Ngroup);']);
  eval(['diff_mat' num2str(kk) ' = zeros(64,64,Ngroup);']);
  eval(['ch' num2str(kk) ' = zeros(64,64);']);
end

% For each exposure level repeat
h = waitbar(0,'Please wait... Processing data');
for ii=1:Nexp,
   % For Ngroup files repeat
    for jj=1:Ngroup,
          % Read images
          I(:) = readraw([dname F{k}],'O');
          % Separate color channels, pick up selected pixel area
          for kk=1:nc,
            eval(['ch' num2str(kk) '(:) = double(I(area_y,area_x,kk));']);
            eval(['chans' num2str(kk) '(:,:,jj) = ch' num2str(kk) ';']);
          end
          k = k+1;
    end

    % Calculate average color components for current exposure level
    for kk=1:nc, eval(['ave_ch' num2str(kk) '(:,:,ii) = mean(chans' num2str(kk) ',3);']); end
    % Calculate pixel-by-pixel STD:s for color components
    for kk=1:nc, eval(['std_ch' num2str(kk) '(:,:,ii) = std(chans' num2str(kk) ',0,3);']); end    
    % Auxiliary matrices
    for kk=1:nc, eval(['tmp_ch' num2str(kk) ' = ave_ch' num2str(kk) '(:,:,ii);']); end        
    % Calculate mean values of average color components at each exposure
    % level and subtract electrical offset level
    for kk=1:nc, mC(ii,kk) = eval(['mean(tmp_ch' num2str(kk) '(:))-offset;']); end            
    % Calculate standard deviations of average color components
    for kk=1:nc, s_ave(ii,kk) = eval(['std(tmp_ch' num2str(kk) '(:));']); end                
    % Difference each image with average image in order to remove fixed
    % pattern noise
    for kk=1:nc, eval(['diff_mat' num2str(kk) '(:) = chans' num2str(kk) '-repmat(tmp_ch' num2str(kk) ',[1 1 Ngroup]);']); end    
    % Reshape difference matrix into matrix with one difference image per column
    for kk=1:nc, eval(['rdiff_mat' num2str(kk) ' = squeeze(reshape(diff_mat' num2str(kk) ',pixarea(1)*pixarea(2),1,Ngroup));']); end
    % Calculate STDs of each differenced image
    for kk=1:nc, eval(['s_diff' num2str(kk) ' = std(rdiff_mat' num2str(kk) ');']); end
    % Calculate means of variances of differenced color components
    for kk=1:nc, var_diff(ii,kk) = eval(['sum(s_diff' num2str(kk) '.^2)/Ngroup;']); end                    
    % Remove quantization noise: var(QN) = 1/12 DN
    %var_diff(ii,:) = var_diff(ii,:)-1/12; 
    % Calculate temporal noise STD and fixed pattern noise STD
    s_temp(ii,:) = sqrt(Ngroup*var_diff(ii,:)/(Ngroup-1)); 
    s_fp(ii,:) = sqrt(s_ave(ii,:).^2-(var_diff(ii,:)/(Ngroup-1)));
    waitbar(ii/Nexp),    
end
close(h);
% Calculate noise levels as percentage of maximum signal level
%if ext=='x3f', signal_max = 10200; 
%else signal_max = 4095; 
%end
s_ave = 100*s_ave/smax; s_temp = 100*s_temp/smax; s_fp = 100*s_fp/smax;

% Calculate signal-to-temporal noise ratio
SNR_temp = 20*log10(mC./s_temp);

% Calculate photoresponse nonuniformity at 50% level
ind = min(find(mC(:,1)>=0.5*smax));
if isempty(ind), [m,ind] = max(mC(:,1)); end
for kk=1:nc, 
  PRNU_ch = eval(['ave_ch' num2str(kk) '(:,:,ind)-ave_ch' num2str(kk) '(:,:,min_ind);']);
  PRNU(1,kk) = 100*std(PRNU_ch(:))/mean(PRNU_ch(:));
end

PRNU_lev = exposure(ind);

out = cell(1,nc*2);
for ii=1:nc, out(ii) = eval(['{ave_ch' num2str(ii) '};']); end
for ii=1:nc, out(nc+ii) = eval(['{std_ch' num2str(ii) '};']); end

