function [val, out] = oecf(dname, ext, exposure, Ngroup, pixarea, vis)

% OECF              - Calculates opto-electronic conversion function (OECF)
%                     according to ISO 14524
%
% Function call:    [val,out] = oecf(dname,ext,exposure,Ngroup,pixarea,vis);
%
% In:   dname       Directory name for data files
%       ext         file name extension (e.g. 'x3f')
%       exposure    Input exposure values [lux*s] or scene log luminance
%                   values [cd*s/m^2]
%       Ngroup      Number of files in one exposure group (default = 10)
%       pixarea     pixel area used for calculation,
%                   [min_row max_row min_col max_col]
%                   (default 64 x 64 pixels in the middle of the image)
%       vis         result plotting; 1 = yes, 0 = no (default 0, use
%                   plot_oecf for results)
%
% Out:  val         logarithmic exposure values
%       out         cell array containg: 
%                   ch_mean1, ch_mean2 etc; means of color channels of average images

% (c) Heli Hytti 26.11.2004
% HH 22.2.2006: arbitrary number of color channels

% Set default parameter values if necessary:
% Result plotting off
if nargin<6, vis = 0; end
% Number of frames per exposure level
if nargin<4, Ngroup = 10; end

% Add backslash to the end of directory name if it's missing
if dname(end)~='\', dname = [dname '\']; end

% Input log illuminance/exposure or scene log luminance
val = log10(exposure);

% Get file names in time order (monotonically increasing exposure level
% assumed)
F = files(dname,ext,'t');

% Read test image
I = readraw([dname F{1}],'O');
nw = size(I,2); nh = size(I,1); nc = size(I,3);

% Number of pixels in the middle of the image used for calculation
if nargin<5,
  area_x = round((nw/2)-31):round((nw/2)+32); 
  area_y = round((nh/2)-31):round((nh/2)+32);
% ... or use the given pixel area
else
  area_x = [pixarea(3):pixarea(4)]; 
  area_y = [pixarea(1):pixarea(2)];
end

% Number of pixels columnwise/rowvise in selected area
lax = length(area_x); lay = length(area_y);

% Find number of image files and calculate number of exposure levels
Ns = size(F,2);
Nexp = Ns/Ngroup;

% Initialize output variables
for ii=1:nc, eval(['ch' num2str(ii) ' = zeros(lay,lax);']); eval(['ch_mean' num2str(ii) ' = zeros(length(val),1);']); end

% Initialize file index k
k = 1;

% For each exposure level repeat
h = waitbar(0,'Please wait... Processing data');
for ii=1:Nexp,
    % Initialize signal matrices
    for kk=1:nc, eval(['chans' num2str(kk) ' = zeros(lay,lax);']); end
    % For Ngroup files repeat
    for jj=1:Ngroup,
        % Read images
        I(:) = readraw([dname F{k}],'O');
        % Separate color channels, pick up selected pixel area, calculate
        % signal sums
        for kk=1:nc,
          eval(['ch' num2str(kk) '(:) = double(I(area_y,area_x,kk));']);
          eval(['chans' num2str(kk) ' = chans' num2str(kk) ' + ch' num2str(kk) ';']);
        end
        k = k+1;
    end

    % Calculate mean signal levels
    for kk=1:nc, eval(['ch_mean' num2str(kk) '(ii) = mean(chans' num2str(kk) '(:))/Ngroup;']); end
    
    waitbar(ii/Nexp),
end
close(h);

out = cell(1,nc);
for ii=1:nc, out(ii) = eval(['{ch_mean' num2str(ii) '};']); end

if vis==1, 
  plot_oecf(ext, val, out); 
end
