function plot_noise_result(exposure,s_ave,s_temp,s_fp,SNR_temp,PRNU,PRNU_lev,arg_in)

% PLOT_NOISE_RESULT   - Plots noise component analysis results
%
% Function call:    
%   plot_noise_result(exposure,s_ave,s_temp,s_fp,SNR_temp,PRNU,PRNU_lev,arg_in);
%
% In:   exposure   Input exposure values [lux*s] os scene log luminance
%                  values [cd*s/m^2]
%       s_ave      standard deviations of average color components (columns)
%       s_temp     standard deviations of temporal noise for color components
%       s_fp       standard deviations of the fixed pattern noise
%       SNR_temp   signal-to-temporal noise ratio at each exposure level
%       PRNU       photo-response non-uniformity
%       PRNU_lev   photoresponse non-uniformity determination level
%       arg_in     cell array containing:
%                  ave_ch1, ave_ch2 etc; 3D-matrices of average color component values (one for each exposure level, 
%                  64 x 64 matrix from the middle of the image)
%                  std_ch1, std_ch2 etc: pixel-by-pixel STDs of color components at each exposure level
%                  (output from noise_analysis)

% (c) Heli Hytti 22.2.2005
% HH 22.2.2006: arbitrary number of color channels

% Input log illuminance/exposure or scene log luminance
val = log10(exposure);

nc = size(arg_in,2)/2;
if nc==1, cnames = 'grayscale'; lcolor = 'k'; lmarker = 'none';
elseif nc==3, cnames = str2mat('red','green','blue'); lcolor = str2mat('r','g','b'); lmarker = str2mat('.','+','x');
elseif nc==4, cnames = str2mat('red','g-red','blue','g-blue'); lcolor = str2mat('r','g','b','g'); lmarker = str2mat('.','+','.','x');
end

% Plot average color components
for ii=1:nc, 
  Im_ave = arg_in{ii}; 
  figure; h = immulti(Im_ave); title(['Average ' deblank(cnames(ii,:)) ' components at each exposure level']); 
end

% Plot pixel-by-pixel STDs
for ii=1:nc, 
  std_ch = arg_in{ii+nc}; 
  figure; h = immulti(std_ch,[0 globmax(std_ch)]); title(['Pixel-by-pixel ' deblank(cnames(ii,:)) ' component STDs at each exposure level']); 
end

% Plot temporal SNRs for each color component
figure;
for ii=1:nc, 
  h = line(val,SNR_temp(:,ii),'linestyle','-','linewidth',2,'color',lcolor(ii,:),'marker',lmarker(ii,:)); 
end
title('Temporal SNRs of color components');
xlabel('log_1_0 exposure level'); ylabel('[dB]');

% Plot STDs of average color components
figure;
for ii=1:nc, 
  h = line(val,s_ave(:,ii),'linestyle','-','linewidth',2,'color',lcolor(ii,:),'marker',lmarker(ii,:)); 
end
title('Standard deviations of average color components');
xlabel('log_1_0 exposure level'); ylabel('[% FSD]');

% Plot temporal noise STDs of color components
figure;
for ii=1:nc, 
  h = line(val,s_temp(:,ii),'linestyle','-','linewidth',2,'color',lcolor(ii,:),'marker',lmarker(ii,:)); 
end
title('Standard deviations of temporal noise');
xlabel('log_1_0 exposure level'); ylabel('[% FSD]');

% Plot fixed pattern noise STDs of color components
figure;
for ii=1:nc, 
  h = line(val,s_fp(:,ii),'linestyle','-','linewidth',2,'color',lcolor(ii,:),'marker',lmarker(ii,:)); 
end
title('Standard deviations of fixed pattern noise');
xlabel('log_1_0 exposure level'); ylabel('[% FSD]');

disp(['Photoresponse non-uniformity for [' reshape(cnames',1,length(cnames(:))) '] is: ']), PRNU,
disp('Signal level for which PRNU was calculated is ([lux*s] or [(cd/m^2)*s] :'), PRNU_lev

