function [S, std_S, std_FPN, K, RN, FWC, DR, offset] = ptc(dname, ext, exposure, Ngroup, pixarea, vis)

% PTC            - Calculates photon transfer curve (PTC)according to
%                  Janesic: "Scientific Charge-Coupled Devices" 
%
% Function call:    
%      [S, std_S, std_FPN, K, RN, FWC, DR, offset] = ptc(dname, ext, exposure, Ngroup, pixarea, vis);
%
% In:   dname       Directory name for data files
%       ext         file name extension (e.g. 'x3f')
%       exposure    Input exposure values [lux*s] or scene log luminance
%                   values [cd*s/m^2]
%       Ngroup      Number of files in one exposure group (default = 10)
%       pixarea     pixel area used for calculation,
%                   [min_row max_row min_col max_col]
%                   (default 64 x 64 pixels in the middle of the image)
%       vis         result plotting; 1 = yes, 0 = no (default 0, use
%                   plot_ptc for results)
%
% Out:  S           Averages of mean red, green and blue digital values for each exposure level
%       std_S       Averages of STDs of differenced red, green and blue
%                   components (temporal noise)
%       std_FPN     Fixed pattern noise components
%       K           estimated camera gain constant [e-/DN] (mean and std
%                   for each color component plus number of estimation points, nc by 3 matrix)
%       RN          Read noise [rms e-] (nc by 1 matrix)
%       FWC         Full well capacity [e-/pixel] (nc by 1 matrix)
%       DR          Dynamic range [dB] (nc by 1 matrix)
%       offset      signal electrical offset value
%
%                   NOTE: nc = number of color components in image

% (c) Heli Hytti 11.1.2005
% HH 21.2.2006: arbitrary number of color channels

% Add backslash to the end of directory name if it's missing
if dname(end)~='\', dname = [dname '\']; end

% Find index for minimum exposure level (dark frames)
[minexp,min_ind] = min(exposure);

% Get file names in time order (monotonically increasing exposure level
% assumed)
F = files(dname,ext,'t');

% Set default parameters if needed:
% Result plotting off
if nargin<6, vis = 0; end

% Read test image
% Get image size and calculate coordinates for 64 x 64 pixel area in the middle
% of the image
I = readraw([dname F{1}],'O');
nw = size(I,2); nh = size(I,1); nc = size(I,3);
if nargin<5,
  area_x = round((nw/2)-31):round((nw/2)+32); 
  area_y = round((nh/2)-31):round((nh/2)+32);
% ... or use the given pixel area
else
  area_x = [pixarea(3):pixarea(4)]; 
  area_y = [pixarea(1):pixarea(2)];
end

if nc==1, cnames = 'grayscale';
elseif nc==3, cnames = str2mat('red','green','blue');
elseif nc==4, cnames = str2mat('red','g-red','blue','g-blue');
end

% Default number of frames per exposure level is 10
if nargin<4, Ngroup = 10; end

% Find number of image files and calculate number of exposure levels
Ns = size(F,2);
Nexp = Ns/Ngroup;

% Number of pixels columnwise/rowvise in selected area
lax = length(area_x); lay = length(area_y);

% Initialize output matrices for signal (S) and noise (std_S) and K
S = zeros(Nexp,nc); std_S = S; std_FPN = S;
% Initialize file index k
k = 1;
for ii=1:nc, eval(['ch' num2str(ii) ' = zeros(lay,lax);']); end

% For each exposure level repeat
h = waitbar(0,'Please wait... Processing data');
for ii=1:Nexp,
    % Initialize signal and difference matrices
    for kk=1:nc, eval(['chans' num2str(kk) ' = zeros(lay,lax,Ngroup);']);
      eval(['diff_mat' num2str(kk) ' = zeros(lay,lax,Ngroup);']);
      eval(['ch_ave' num2str(kk) ' = zeros(lay,lax);']);
    end
    % For Ngroup files repeat
    for jj=1:Ngroup,
        % Read images
        I = readraw([dname F{k}],'O');
        % Separate color channels, pick up selected pixel area
        for kk=1:nc,
          eval(['ch' num2str(kk) ' = double(I(area_y,area_x,kk));']);
          eval(['chans' num2str(kk) '(:,:,jj) = ch' num2str(kk) ';']);
        end
        k = k+1;
    end
	
    % Calculate average color components
    for kk=1:nc, eval(['ch_ave' num2str(kk) ' = mean(chans' num2str(kk) ',3);']); end

    % Calculate standard deviations of average color components
    s_ave = zeros(1,nc);
    for kk=1:nc, s_ave(1,kk) = eval(['std(ch_ave' num2str(kk) '(:));']); end
    
    % Difference each image with average image in order to remove fixed
    % pattern noise
    for kk=1:nc, eval(['diff_mat' num2str(kk) '(:) = chans' num2str(kk) '-repmat(ch_ave' num2str(kk) ',[1 1 Ngroup]);']); end

    % Reshape difference matrix into matrix with one difference image per column
    for kk=1:nc, eval(['rdiff_mat' num2str(kk) ' = squeeze(reshape(diff_mat' num2str(kk) ',lay*lax,1,Ngroup));']); end

    % Calculate mean signal level S (PTC x axis)
    for kk=1:nc, S(ii,kk) = eval(['mean(ch_ave' num2str(kk) '(:));']); end

    % Calculate variances of each differenced image
    for kk=1:nc, eval(['v_diff' num2str(kk) ' = var(rdiff_mat' num2str(kk) ');']); end
    
    % Calculate average variance and STD of differenced frames (PTC y axis)
    var_S = zeros(1,nc);
    for kk=1:nc, var_S(1,kk) = eval(['sum(v_diff' num2str(kk) ')/Ngroup;']); end

    % Temporal noise STD and fixed pattern noise STD
    std_S(ii,:) = sqrt(Ngroup*var_S/(Ngroup-1));
    std_FPN(ii,:) = sqrt(s_ave.^2-(var_S/(Ngroup-1)));
    waitbar(ii/Nexp)    
end
close(h);
% Find electrical offset (This is still a bit gray area; assume that 
% the dark exposure signal level is the offset. 0.1 is added to avoid log of zero.)
offset = min(S(:))-0.1;
% Subtract offset
S = S-offset;
% Estimate camera gain constant K mean and std for each color component
% Shot noise has to exceed the read noise floor
% If any of the estimated K values for each component is more than twice
% the minimum value, they are eliminated
for kk=1:nc,
  eval(['ind_' num2str(kk) '= find(std_S(:,kk)>=2*std_S(min_ind,kk) & std_S(:,kk)>0);']);
  eval(['K' num2str(kk) ' = S(ind_' num2str(kk) ',kk)./(std_S(ind_' num2str(kk) ',kk).^2-std_S(min_ind,kk)^2);']);
  eval(['indK' num2str(kk) ' = find(K' num2str(kk) '>=2*min(K' num2str(kk) ')); if ~isempty(indK' num2str(kk) '), K' num2str(kk) '(indK' num2str(kk) ') = []; ind_' num2str(kk) '(indK' num2str(kk) ') = []; end']);
end

K = zeros(nc,3);
for kk=1:nc,
   K(kk,:) = eval(['[mean(K' num2str(kk) ') std(K' num2str(kk) ') length(ind_' num2str(kk) ')];']);
end

% Calculate read noise based on minimum noise and K, remove quantization
% noise (1/12 DN)
RN = K(:,1).*sqrt(std_S(min_ind,:).^2-(1/12))';

% Calculate full well capacity based on maximum signal and K
% (This is not a correct definition) 
FWC = K(:,1).*max(S)';
% Calculate dynamic range by FWC (or maximum signal) and read noise
DR = 20*log10(FWC./RN); 

% Plot results if requested
if vis==1, plot_ptc(ext, S, std_S, std_FPN);
disp(['Camera gain constant K mean, std and number of calculation points for [' reshape(cnames',1,length(cnames(:))) ']: ']), K,
disp(['Read noise as electrons for [' reshape(cnames',1,length(cnames(:))) ']: ']), RN,
disp('Full well capacity based on maximum signal level: '), FWC,
disp('Dynamic range as dB: '), DR,
end
