function [M SumSQResid] = generate_polyparam(method, IRfilter, showplots)
        
% showplots :  whether it shows a suitably scaled version of the picture being
%            converted. The default is "show nothing" (showplots = 0)
% IRfilter : whether the IR-blocking filter was used in during the image
%            capture (used for bakwards compatibility with databases
%            gathered WITHOUT the filter). The default is FILTER ON (IRfilter = 1)
% method   : this determines the choice of polynomial coversion.
%            There are seveal flavours to date (see the test_all_poly_options,
%            but the most common use of this code is to convert the pictures to 
%            the Stockmann & Sharpe LMS fundamentals using the datset of Munsell 
%            chip reflectances as a training set (method = SS_LMS_Munsell)
%            or to convert them to the standard CIE1931 XYZ tristimulus
%            values using the datset of Munsell chip reflectances as a training 
%            set (method = CIE1931_XYZ_Munsell). The complete set of options are: 
%            SP_LMS_Munsell: Smith & Pokorny LMS (normalised to max=1) and Munsell chips 
%            SP_LMS_NE     : Smith & Pokorny LMS (normalised to max=1) and NE reflectances
%            SS_LMS_Munsell: Stockmann & Sharpe 2000 LMS (normalised to max=1) and Munsell chips
%            SS_LMS_NE     : Stockmann & Sharpe 2000 LMS (normalised to max=1) and NE reflectances 
%            CIE1931_XYZ_Munsell : CIE1931 XYZ (normalised to max=1) and Munsell chips
%            CIE1931_XYZ_NE      : CIE1931 XYZ (normalised to max=1) and NE reflectances
%            JV1978_XYZ_Munsell  : Judd-Voss 1978 XYZ (normalised to max=1) and Munsell chips
%            JV1978_XYZ_NE       : Judd-Voss 1978 XYZ (normalised to max=1)
%            and NE reflectances


if nargin < 3
    showplots = 0; %do not show correlation plots
end

if nargin < 2 || isempty(IRfilter)
    IRfilter = 1; %IR-filter is ON by default
end

if (nargin < 1) || isempty(method)
    method = 'cie1931_xyz_munsell';
end

method = lower(method);
load ('sigma_foveon.mat'); %sensors
temp = 6500;    %D65 = default illuminant
interval = [400 700];

if IRfilter    
    load('BW_IR486.mat'); %IRblocking 
else
    IRblocking = [(400:700)',ones(700-400+1,1)];
end
    
k = 1; %no 

switch method 
    case 'sp_lms_munsell' %
        disp('Smith & Pokorny LMS (normalised to max=1) and Munsell chips'); 
        load('munsell1269.mat');    %reflec
        load('cone_fundamentals'); %LMS_SmithPokorny_unitymax_1nm
        temp = 6500;    %D65 = default illuminant
        FoveonRGB = sensors_capture(sensors_normalised, reflec, temp, interval, IRblocking);
        Groundtruth = sensors_capture(LMS_SmithPokorny_unitymax_1nm, reflec, temp, interval, 0, k);
    case 'sp_lms_ne'      
        disp('Smith & Pokorny LMS (normalised to max=1) and NE reflectances')
        load('NEurop219.mat');    %reflec
        load('cone_fundamentals'); %LMS_SmithPokorny_unitymax_1nm
        temp = [5000 5500 6500 7000 9000 10000 11000]; 
        % Typical daylight; The sun at noon near Kodak's offices; D65; Slightly overcast sky;
        %  Open shade on clear day; Heavily overcast sky; Sunless blue skies
        FoveonRGB = sensors_capture(sensors_normalised, reflec, temp, interval, IRblocking);
        Groundtruth = sensors_capture(LMS_SmithPokorny_unitymax_1nm, reflec, temp, interval, 0, k);        
    case 'ss_lms_munsell' 
        disp('Stockmann & Sharpe 2000 LMS (normalised to max=1) and Munsell chips');
        load('munsell1269.mat');    %reflec
        load('cone_fundamentals'); %LMS_StockmanSharpe_unitymax_1nm
        temp = 6500;    %D65 = default illuminant
        FoveonRGB = sensors_capture(sensors_normalised, reflec, temp, interval, IRblocking);
        Groundtruth = sensors_capture(LMS_StockmanSharpe_unitymax_1nm, reflec, temp, interval, 0, k);        
    case 'ss_lms_ne'      
        disp('Stockmann & Sharpe 2000 LMS (normalised to max=1) and NE reflectances');
        load('NEurop219.mat');    %reflec
        load('cone_fundamentals'); %LMS_StockmanSharpe_unitymax_1nm
        temp = [5000 5500 6500 7000 9000 10000 11000]; 
        % Typical daylight; The sun at noon near Kodak's offices; D65; Slightly overcast sky;
        %  Open shade on clear day; Heavily overcast sky; Sunless blue skies
        FoveonRGB = sensors_capture(sensors_normalised, reflec, temp, interval, IRblocking);
        Groundtruth = sensors_capture(LMS_StockmanSharpe_unitymax_1nm, reflec, temp, interval, 0, k);
    case 'cie1931_xyz_munsell' 
        disp('CIE1931 XYZ (normalised to max=1) and Munsell chips');
        load('munsell1269.mat');    %reflec
        load('colour_matching_functions.mat'); %XYZCIE1931_1nm
        temp = 6500;    %D65 = default illuminant
        FoveonRGB = sensors_capture(sensors_normalised, reflec, temp, interval, IRblocking);
        Groundtruth = sensors_capture(XYZCIE1931_1nm, reflec, temp, interval, 0, k);
    case 'cie1931_xyz_ne'      
        disp('CIE1931 XYZ (normalised to max=1) and NE reflectances');
        load('NEurop219.mat');    %reflec
        load('colour_matching_functions.mat'); %XYZCIE1931_1nm
        temp = [5000 5500 6500 7000 9000 10000 11000];
        FoveonRGB = sensors_capture(sensors_normalised, reflec, temp, interval, IRblocking);
        Groundtruth = sensors_capture(XYZCIE1931_1nm, reflec, temp, interval, 0, k);
    case 'jv1978_xyz_munsell'  
        disp('Judd-Voss 1978 XYZ (normalised to max=1) and Munsell chips');
        load('munsell1269.mat');    %reflec
        load('colour_matching_functions.mat'); %XYZjuddvoss_1nm
        temp = 6500;    %D65 = default illuminant
        FoveonRGB = sensors_capture(sensors_normalised, reflec, temp, interval, IRblocking);
        Groundtruth = sensors_capture(XYZjuddvoss_1nm, reflec, temp, interval, 0, k);
    case 'jv1978_xyz_ne'       
        disp('Judd-Voss 1978 XYZ (normalised to max=1) and NE reflectances');
        load('NEurop219.mat');    %reflec
        load('colour_matching_functions.mat'); %XYZjuddvoss_1nm
        temp = [5000 5500 6500 7000 9000 10000 11000];
        FoveonRGB = sensors_capture(sensors_normalised, reflec, temp, interval, IRblocking);
        Groundtruth = sensors_capture(XYZjuddvoss_1nm, reflec, temp, interval, 0, k);
    otherwise
        error('Incorrect method [args: sp_lms_munsell, sp_lms_ne, ss_lms_munsell, ss_lms_ne, cie1931_xyz_munsell, cie1931_xyz_ne, jv1978_xyz_munsell, jv1978_xyz_nemunsell_s&plms, ne_s&plms, munsell_s&slms, ne_s&slms, munsell_cie1931, ne_cie1931, munsell_jussvoss, ne_juddvoss]');
 end

[M, SumSQResid] = PolynomialCoefficients(FoveonRGB, Groundtruth, showplots);
